//
//  RobotCommunication.m
//  Macoun2013
//
//  Created by Frank Jüstel on 03.10.13.
//  Copyright (c) 2013 JSM Arts Webservices GbR. All rights reserved.
//

#import <AVFoundation/AVFoundation.h>

#import "RoboTalker.h"

// Für die Sockets
#import "GCDAsyncSocket.h"

// Wir sind höflich und geben jedem Send einen Tag mit auf den Weg
enum {
	DataWriteTagInitSocket,
	DataWriteTagBackward,
	DataWriteTagLeft,
	DataWriteTagRight,
	DataWriteTagForward,
};
typedef long DataWriteTag;

@interface RoboTalker () <GCDAsyncSocketDelegate>
@property GCDAsyncSocket* socket;
@property AVSpeechSynthesizer* speechSynthesizer;
@end

@implementation RoboTalker

+(instancetype)sharedInstance
{
	static RoboTalker* sharedInstance;
	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^{
		sharedInstance = [[self alloc] init];
	});
	return sharedInstance;
}

-(instancetype) init
{
	if (self = [super init]) {
		// Socket initialisieren.
		dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
		self.socket = [[GCDAsyncSocket alloc] initWithDelegate:self delegateQueue:queue];
		self.speechSynthesizer = [[AVSpeechSynthesizer alloc] init];
	}
	return self;
}

-(void) connect
{
	if (!self.isConnected) {
		NSError* error;
		[self.socket connectToHost:ROBOT_IP_ADDRESS onPort:ROBOT_PORT error:&error];
		[error dump];
	}
}

-(void)disconnect
{
	if (self.isConnected) {
		[self.socket disconnect];
	}
}

-(BOOL)isConnected
{
	return self.socket.isConnected;
}

-(void) sendForward
{
	if (!self.isConnected) return;
	
	NSDictionary* message = @{@"type" : @"direction", @"direction" : @"forward"};
	
	NSError* error;
	NSData* messageJSON = [NSJSONSerialization dataWithJSONObject:message options:0 error:&error];
	[error dump];
	
	[self.socket writeData:messageJSON withTimeout:-1 tag:DataWriteTagForward];
	
	AVSpeechUtterance* utterance = [AVSpeechUtterance speechUtteranceWithString:@"forward"];
	utterance.voice = [AVSpeechSynthesisVoice voiceWithLanguage:@"en-US"];
	
	[self.speechSynthesizer speakUtterance:utterance];
}

-(void) sendRight
{
	if (!self.isConnected) return;
	
	NSDictionary* message = @{@"type" : @"direction", @"direction" : @"right"};
	
	NSError* error;
	NSData* messageJSON = [NSJSONSerialization dataWithJSONObject:message options:0 error:&error];
	[error dump];
	
	[self.socket writeData:messageJSON withTimeout:-1 tag:DataWriteTagRight];

	AVSpeechUtterance* utterance = [AVSpeechUtterance speechUtteranceWithString:@"right"];
	utterance.voice = [AVSpeechSynthesisVoice voiceWithLanguage:@"en-US"];

	[self.speechSynthesizer speakUtterance:utterance];
}

-(void) sendLeft
{
	if (!self.isConnected) return;
	
	NSDictionary* message = @{@"type" : @"direction", @"direction" : @"left"};
	
	NSError* error;
	NSData* messageJSON = [NSJSONSerialization dataWithJSONObject:message options:0 error:&error];
	[error dump];
	
	[self.socket writeData:messageJSON withTimeout:-1 tag:DataWriteTagLeft];

	AVSpeechUtterance* utterance = [AVSpeechUtterance speechUtteranceWithString:@"left"];
	utterance.voice = [AVSpeechSynthesisVoice voiceWithLanguage:@"en-US"];
	
	[self.speechSynthesizer speakUtterance:utterance];
}

-(void) sendBackward
{
	if (!self.isConnected) return;
	
	NSDictionary* message = @{@"type" : @"direction", @"direction" : @"backward"};
	
	NSError* error;
	NSData* messageJSON = [NSJSONSerialization dataWithJSONObject:message options:0 error:&error];
	[error dump];
	
	[self.socket writeData:messageJSON withTimeout:-1 tag:DataWriteTagBackward];

	AVSpeechUtterance* utterance = [AVSpeechUtterance speechUtteranceWithString:@"backward"];
	utterance.voice = [AVSpeechSynthesisVoice voiceWithLanguage:@"en-US"];
	
	[self.speechSynthesizer speakUtterance:utterance];
}

#pragma mark - GCDAsyncSocketDelegate

// Wenn wir eine Verbindung zum FlyPort haben, sagen wir freundlich guten Tag :-)
-(void)socket:(GCDAsyncSocket *)sock didConnectToHost:(NSString *)host port:(uint16_t)port
{
	NSLog(@"%s", __PRETTY_FUNCTION__);
	NSDictionary* message = @{@"type" : @"init", @"config" : @"none"};
	
	NSError* error;
	NSData* messageJSON = [NSJSONSerialization dataWithJSONObject:message options:0 error:&error];
	[error dump];
	
	[self.socket writeData:messageJSON withTimeout:-1 tag:DataWriteTagInitSocket];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"socketDidConnect" object:self.socket];
	
	AVSpeechUtterance* utterance = [AVSpeechUtterance speechUtteranceWithString:@"connected"];
	utterance.voice = [AVSpeechSynthesisVoice voiceWithLanguage:@"en-US"];
	
	[self.speechSynthesizer speakUtterance:utterance];
}

-(void)socketDidDisconnect:(GCDAsyncSocket *)sock withError:(NSError *)err
{
	NSLog(@"%s", __PRETTY_FUNCTION__);
	[[NSNotificationCenter defaultCenter] postNotificationName:@"socketDidDisconnect" object:self.socket];

	AVSpeechUtterance* utterance = [AVSpeechUtterance speechUtteranceWithString:@"diconnected"];
	utterance.voice = [AVSpeechSynthesisVoice voiceWithLanguage:@"en-US"];
	
	[self.speechSynthesizer speakUtterance:utterance];
}

-(void)socket:(GCDAsyncSocket *)sock didWriteDataWithTag:(long)tag
{
	NSLog(@"didWriteDataWithTag %ld", tag);
}

@end
