//
//  ViewController.m
//  Macoun2013
//
//  Created by Frank Jüstel on 01.10.13.
//  Copyright (c) 2013 JSM Arts Webservices GbR. All rights reserved.
//

#import "ViewController.h"
#import "RoboTalker.h"

@interface ViewController ()
@property (weak, nonatomic) IBOutlet UIImageView *backgroundImageViewOutlet;
@property (weak, nonatomic) IBOutlet UIButton *connectionButtonOutlet;
@end

@implementation ViewController

#pragma mark - view lifecycle
- (void)viewDidLoad
{
    [super viewDidLoad];

	self.backgroundImageViewOutlet.image = [[self.backgroundImageViewOutlet.image blur] scaleIfNeeded];
	
	// Zunächst einmal sind wir nicht connected, also sind auch keine Richtungsknöpfe aktiv
	[self enableDirectionButtons:NO];
	
	// Der ViewController ist der BasisController, er kann sich nie abbauen und dehalb müssen wir uns auch die id's der oberserver nicht merken, da wir hier sowieso nicht in den dealloc laufen würden. Aber sonst muss man so etwas natürlich machen (siehe BrainViewController)
	[[NSNotificationCenter defaultCenter] addObserverForName:@"socketDidConnect" object:nil queue:nil usingBlock:^(NSNotification *note)
	{
		dispatch_async(dispatch_get_main_queue(), ^{
			self.connectionButtonOutlet.selected = YES;
			self.connectionButtonOutlet.enabled = YES;
			[self enableDirectionButtons:YES];
		});
	}];
	
	[[NSNotificationCenter defaultCenter] addObserverForName:@"socketDidDisconnect" object:nil queue:nil usingBlock:^(NSNotification *note)
	{
		dispatch_async(dispatch_get_main_queue(), ^{
			self.connectionButtonOutlet.selected = NO;
			self.connectionButtonOutlet.enabled = YES;
			[self enableDirectionButtons:NO];
		});
	}];
}

#pragma mark - IBActions
- (IBAction)forwardButtonTouchUpInside:(id)sender
{
	[[RoboTalker sharedInstance] sendForward];
}

- (IBAction)rightButtonTouchUpInside:(id)sender
{
	[[RoboTalker sharedInstance] sendRight];
}

- (IBAction)leftButtonTouchUpInside:(id)sender
{
	[[RoboTalker sharedInstance] sendLeft];
}

- (IBAction)backButtonTouchUpInside:(id)sender
{
	[[RoboTalker sharedInstance] sendBackward];
}

- (IBAction)connectionButtonTouchUpInside:(id)sender
{
	self.connectionButtonOutlet.enabled = NO;
	if (![RoboTalker sharedInstance].isConnected) {
		[[RoboTalker sharedInstance] connect];
	} else {
		[[RoboTalker sharedInstance] disconnect];
	}
}

#pragma mark - private methods

- (void) enableDirectionButtons: (BOOL) yesOrNo
{
	dispatch_async(dispatch_get_main_queue(), ^{
		for (id view in self.view.subviews) {
			if (view == self.connectionButtonOutlet) continue;
			[view setEnabled:yesOrNo];
		}
	});
}
@end
